@@ -10,7 +10,7 @@
 #------------------------------.onLoad-------------------------------
 .onLoad <- function(...){
   lib <- dirname(system.file(package = "mice"))
-  d <- packageDescription("mice", lib = lib)
+  d <- packageDescription("mice", lib.loc = lib)
   cat(paste(d$Package,d$Version,d$Date,"\n"))
 
   require(MASS, quietly=TRUE)
@@ -20,6 +20,6 @@
 
 version <- function(pkg="mice"){
   lib <- dirname(system.file(package = pkg))
-  d <- packageDescription(pkg, lib = lib)
+  d <- packageDescription(pkg, lib.loc = lib)
   return(paste(d$Package,d$Version,d$Date,lib))
 }
